CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
153 lines
4.0 KiB
C++
153 lines
4.0 KiB
C++
// Copyright (c) Advanced Micro Devices, Inc., or its affiliates.
|
|
// SPDX-License-Identifier: MIT
|
|
|
|
#ifndef CK_CONTAINER_ELEMENT_PICKER_HPP
|
|
#define CK_CONTAINER_ELEMENT_PICKER_HPP
|
|
|
|
#include "functional2.hpp"
|
|
#include "sequence.hpp"
|
|
|
|
namespace ck {
|
|
|
|
// Arr: Array or StaticallyIndexedArray
|
|
// Picks: Sequence<...>
|
|
template <typename Arr, typename Picks>
|
|
struct ContainerElementPicker
|
|
{
|
|
using type = ContainerElementPicker;
|
|
|
|
__host__ __device__ constexpr ContainerElementPicker() = delete;
|
|
|
|
__host__ __device__ constexpr ContainerElementPicker(Arr& array) : mArray{array}
|
|
{
|
|
constexpr index_t imax =
|
|
reduce_on_sequence(Picks{}, math::maximize<index_t>{}, Number<0>{});
|
|
|
|
static_assert(imax < Arr::Size(), "wrong! exceeding # array element");
|
|
}
|
|
|
|
__host__ __device__ static constexpr auto Size() { return Picks::Size(); }
|
|
|
|
template <index_t I>
|
|
__host__ __device__ constexpr const auto& At(Number<I> i) const
|
|
{
|
|
static_assert(I < Size(), "wrong!");
|
|
|
|
constexpr auto IP = Picks{}[i];
|
|
return mArray[IP];
|
|
}
|
|
|
|
template <index_t I>
|
|
__host__ __device__ constexpr auto& At(Number<I> i)
|
|
{
|
|
static_assert(I < Size(), "wrong!");
|
|
|
|
constexpr auto IP = Picks{}[i];
|
|
return mArray(IP);
|
|
}
|
|
|
|
template <index_t I>
|
|
__host__ __device__ constexpr const auto& operator[](Number<I> i) const
|
|
{
|
|
return At(i);
|
|
}
|
|
|
|
template <index_t I>
|
|
__host__ __device__ constexpr auto& operator()(Number<I> i)
|
|
{
|
|
return At(i);
|
|
}
|
|
|
|
template <typename T>
|
|
__host__ __device__ constexpr auto operator=(const T& a)
|
|
{
|
|
static_assert(T::Size() == Size(), "wrong! size not the same");
|
|
|
|
static_for<0, Size(), 1>{}([&](auto i) { operator()(i) = a[i]; });
|
|
|
|
return *this;
|
|
}
|
|
|
|
private:
|
|
Arr& mArray;
|
|
};
|
|
|
|
// Arr: Array or StaticallyIndexedArray
|
|
// Picks: Sequence<...>
|
|
template <typename Arr, typename Picks>
|
|
struct ConstantContainerElementPicker
|
|
{
|
|
using type = ConstantContainerElementPicker;
|
|
|
|
__host__ __device__ constexpr ConstantContainerElementPicker() = delete;
|
|
|
|
__host__ __device__ constexpr ConstantContainerElementPicker(const Arr& array) : mArray{array}
|
|
{
|
|
constexpr index_t imax =
|
|
reduce_on_sequence(Picks{}, math::maximize<index_t>{}, Number<0>{});
|
|
|
|
static_assert(imax < Arr::Size(), "wrong! exceeding # array element");
|
|
}
|
|
|
|
__host__ __device__ static constexpr auto Size() { return Picks::Size(); }
|
|
|
|
template <index_t I>
|
|
__host__ __device__ constexpr const auto& At(Number<I> i) const
|
|
{
|
|
static_assert(I < Size(), "wrong!");
|
|
|
|
constexpr auto IP = Picks{}[i];
|
|
return mArray[IP];
|
|
}
|
|
|
|
template <index_t I>
|
|
__host__ __device__ constexpr const auto& operator[](Number<I> i) const
|
|
{
|
|
return At(i);
|
|
}
|
|
|
|
private:
|
|
const Arr& mArray;
|
|
};
|
|
|
|
template <typename Arr, typename Picks, typename X>
|
|
__host__ __device__ constexpr auto operator+=(ContainerElementPicker<Arr, Picks>& y, const X& x)
|
|
{
|
|
using Y = ContainerElementPicker<Arr, Picks>;
|
|
constexpr index_t nsize = Y::Size();
|
|
|
|
static_assert(nsize == X::Size(), "wrong! size not the same");
|
|
|
|
static_for<0, nsize, 1>{}([&](auto i) { y(i) += x[i]; });
|
|
|
|
return y;
|
|
}
|
|
|
|
template <typename Arr, typename Picks, typename X>
|
|
__host__ __device__ constexpr auto operator-=(ContainerElementPicker<Arr, Picks>& y, const X& x)
|
|
{
|
|
using Y = ContainerElementPicker<Arr, Picks>;
|
|
constexpr index_t nsize = Y::Size();
|
|
|
|
static_assert(nsize == X::Size(), "wrong! size not the same");
|
|
|
|
static_for<0, nsize, 1>{}([&](auto i) { y(i) -= x[i]; });
|
|
|
|
return y;
|
|
}
|
|
|
|
template <typename Arr, typename Picks>
|
|
__host__ __device__ constexpr auto pick_container_element(Arr& a, Picks)
|
|
{
|
|
return ContainerElementPicker<Arr, Picks>(a);
|
|
}
|
|
|
|
template <typename Arr, typename Picks>
|
|
__host__ __device__ constexpr auto pick_container_element(const Arr& a, Picks)
|
|
{
|
|
return ConstantContainerElementPicker<Arr, Picks>(a);
|
|
}
|
|
|
|
} // namespace ck
|
|
#endif
|